#ifndef AGI_H
#define AGI_H

class Thread;
class TranslationEntry;

class AutoGraderInterface {
public:
  // projects 1-4 common interface

  /* Call whenever a thread moves to the ready state.
   */

  static void ReadyThread(Thread* thread);

  /* Call whenever a thread moves to the running state.
   */

  static void RunningThread(Thread* thread);

  /* Call whenever a thread finishes.
   */

  static void FinishingThread(Thread* thread);

  // projects 2-4 common interface

  /* Call from UserProgKernel::Run() to determine which process the kernel
   * should run first.
   */

  static char* ShellProgram();

  // projects 3-4 common interface

  /* Call from the exception handler when it is determined that the current TLB
   * fault is actually a page fault. This should be called from the thread that
   * caused the page fault.
   */

  static void PageFault(unsigned vaddr);

  // for internal use only

  static void Init(int argc, char** argv);

  // 1+
  static void Tick(long long when);
  static void Interrupt(int type, long long when);
  static void ConfirmTimerConfiguration(bool random);
  // 2+
  static bool ExceptionHandler(int which);
  static bool PollConsole();
  static char ReadConsole();
  static void WriteConsole(char c);
  // 3+
  static void TLBUpdate(int i, TranslationEntry* entry);
  // 4+
  static void ConfirmNetworkReliability(double reliability);

private:
  static void ParseCommandLine(int argc, char** argv);
  static void Cleanup(int s);

  static char* DebugFlags;
  static char* AGICommand;

};

#endif
