#include "stdio.h"
#include "stdlib.h"

#define BUFFERSIZE 64

/*  --------------------------------------------------------
 *  shell.c
 *
 *  This is a shell program that outputs the nachos prompt
 *  and takes in what is typed, and executes it.
 *  ----------------------------------------------------- */

/* infinate loop to keep displaying the prompt and to try
 * to execute what is typed into the input.
 */

/* acquired by background threads and by main thread when process forked */
Lock displaylock;

struct bgprocessinfo
{
  SpaceId pid;
  char command[BUFFERSIZE];
};

// print a message when a background process completes
void bgexec(struct bgprocessinfo *info)
{
  Join(info->pid);

  AcquireLock(displaylock);

  printf("\nDone	%s\n", info->command);

  ReleaseLock(displaylock);

  free((void*) info);
}

/* deal with a background process */
void startbgexec(SpaceId pid, char* command)
{
  struct bgprocessinfo *info;
  char temp[15];

  info = (struct bgprocessinfo *) malloc(sizeof(struct bgprocessinfo));

  info->pid = pid;
  strcpy(info->command, command);

  ThreadForkWithArg((void (*)(int)) bgexec, (int) info);
}  

/* execute a single command line */
void runline(char* line)
{
  unsigned len,i;
  bool bg=false;
  SpaceId pid;
   
  len = strlen(line);

  /* remove trailing whitespace */

  while (len>0 &&
	 (line[len-1] == ' ' || line[len-1] == '\t'))
    len--;

  if (len>0 && line[len-1] == '&') {
    len--;
    bg=true;
  }
    
  if (len==0)
    return;

  line[len]=0;

  pid = Exec(line);
  if (pid==0) {
    printf("Command not found\n");
    return;
  }

  if (bg) {
    startbgexec(pid,line);
  }
  else {
    AcquireLock(displaylock);
    Join(pid);
    ReleaseLock(displaylock);
  }
}

int
main(int, char**)
{
  char prompt[64] = "nachos% ";
  
  SpaceId newProc;
  char buffer[BUFFERSIZE];
  int i, echo;
  char* cmdline;
  char oldBuffer[BUFFERSIZE];

  displaylock = AllocLock();

  while (1) {
    printf("%s", prompt);

    i = 0;
	
    do {
      buffer[i] = getch();			// get the next character
    }
    while (buffer[i++] != '\n' && i<BUFFERSIZE);// keep reading until you
                                                // hit a newline

    if (buffer[--i] != '\n') {                  // restart if command is too
                                                // long
      printf("Command too long\n");
      continue;
    }
    
    buffer[i] = '\0';                           // append a null terminator

    if (i>0) {
      if (echo)
	printf("%s\n", buffer);                 // echo what was typed
      
      if (strcmp(buffer, "exit")==0)            // exit if exit was typed
	ExitProgram(0);
      if (strcmp(buffer, "!!") == 0)
	strcpy(buffer, oldBuffer);
      else
	cmdline = buffer;

      runline(cmdline);
      strcpy(oldBuffer, buffer);
    }
  }
}




