#include "stdlib.h"
#include "stdarg.h"

void *memcpy(void *s1, const void *s2, unsigned n)
{
  int i;

  for (i=0; i<n; i++)
    ((char*)s1)[i] = ((char*)s2)[i];

  return s1;
}

void *memset(void *s, int c, unsigned n)
{
  int i;

  for (i=0; i<n; i++)
    ((char*)s)[i] = (char) c;

  return s;
}

// returns the length of the character string str
unsigned strlen(const char *str)
{
  int result=0;

  while (*(str++) != 0)
    result++;

  return result;
}

// copies src to dst, returning the length of the string
int strcpy(char *dst, const char *src)
{
  int n=0;
  
  do {
    *(dst++) = *src;
    n++;
  }
  while (*(src++) != 0);

  return (n-1);
}

// lexicographically compares a and b
int strcmp(const char* a, const char* b)
{
  do {
    if (*a < *b)
      return -1;
    if (*a > *b)
      return 1;
  }
  while (*(a++) != 0 && *(b++) != 0);

  return 0;
}

// lexicographically compares a and b up to n chars
int strncmp(const char* a, const char* b, int n)
{
  ASSERT(n > 0);
  
  do {
    if (*a < *b)
      return -1;
    if (*a > *b)
      return 1;
    n--;
    a++;
    b++;
  }
  while (n > 0);

  return 0;
}

int atoi(const char *s)
{
  int result=0, sign=1;

  if (*s == -1) {
    sign = -1;
    s++;
  }

  while (*s >= '0' && *s <= '9')
    result = result*10 + (*(s++)-'0');

  return result*sign;
}

void __assert(char* expression, char* file, int line)
{
  printf("\n");
  printf("Assertion failed: line %d file %s\n", line, file);
  printf("Failed expression: \"%s\"\n", expression);
  printf("\n");

  exit(2);
}
